# -*- coding: UTF-8 -*-

from nes_common.inserter import Inserter 
from cs.mysql.nes_exec import Exec
import MySQLdb
import ConfigParser
import sys
from nes_common import logger
from nes_command import Command

class DB_Inserter(Inserter):

    dbVersionIsOK = {}

    def __init__(self, shop, loadDictStatusSender=None):
        log = logger.Logger().getLogger(shop.get('shopId', ''))
        config = ConfigParser.RawConfigParser()
        if not config.read(sys.path[0] + "/../../../config/nes_load.ini"):
            log.error("Configuration file 'nes_load.ini' not found.")
            sys.exit(1)
        host = config.get('mysql', 'host')
        port = config.getint('mysql', 'port')
        user = config.get('mysql', 'user')
        password = config.get('mysql', 'password')
        self.migrateVersion = config.getint('mysql', 'migrateVersion')
        suffix = shop.get('shopId', '')
        if shop.get('shopId', '').endswith('All') and shop.get('shopId', '').startswith(shop.get('clusterId', '')):
            suffix = 'All'
        executor = Exec(host=host, port=port, user=user, password=password, database='%sdictionaries%s' % (shop.get('clusterId', ''), suffix), shopCode=shop.get('shopId', ''), config=config, loadDictStatusSender=loadDictStatusSender)
        Inserter.__init__(self, shop.get('shopId', ''), executor)
        self.mysqlConnBlock = MySQLdb.connect(host=host, user=user, passwd=password, db='artixcs_struct', port=port)
        self.dbForCheckVersion = '%sartixcs%s' % (shop.get('clusterId', ''), suffix)
        self.mysqlConnDBVersion = MySQLdb.connect(host=host, user=user, passwd=password, db=self.dbForCheckVersion, port=port)
        if not self.dbForCheckVersion in self.dbVersionIsOK:
            self.dbVersionIsOK[self.dbForCheckVersion] = False

    def testDBVersion(self):
        if self.dbVersionIsOK.get(self.dbForCheckVersion, False):
            return True
        try:
            cur = self.mysqlConnDBVersion.cursor()
            cur.execute("select version from migrate_version")
            row = cur.fetchone()
            dbVersion = row[0] if row != None else self.migrateVersion
            cur.execute("commit")
            if dbVersion >= self.migrateVersion:
                self.dbVersionIsOK[self.dbForCheckVersion] = True
                return True
        except:
            return False
        finally:
            cur.close()
        return False

    def flagBlockSet(self):
        cur = self.mysqlConnBlock.cursor()
        self.log.info('Blok storeId=%s' % self.shopCode)
        try:
            cur.execute("start transaction")
            res = cur.execute("insert into extlocktable (storeId,lockedBy) values ( '%s', 'NES')" % self.shopCode)
            cur.execute("commit")
            if res == 1:
                return 1
        except:
            return 0
        finally:
            cur.close()
        return 0

    def testFlagBlockSet(self):
        cur = self.mysqlConnBlock.cursor()
        cur.execute("select * from extlocktable where storeId = '%s' and lockedBy = 'NES'" % self.shopCode)
        if cur.fetchone() != None:
            cur.close()
            return True
        cur.close()
        return False

    def flagUnBlockSet(self):
        cur = self.mysqlConnBlock.cursor()
        self.log.info('UnBlok storeId=%s' % self.shopCode)
        cur.execute("start transaction")
        cur.execute("delete from extlocktable where storeId = '%s' and lockedBy = 'NES'" % self.shopCode)
        cur.execute("commit")
        cur.close()
        if self.testFlagBlockSet():
            return 0
        return 1
    
    def getCommand(self, simpleCommands):
        return Command(self.executor, self.shopCode, simpleCommands = simpleCommands, mode='CS')

