'''
Created on 12.04.2010

@author: gsu
'''

from objects import *
from objects.extend_objects import *
from nes_common import logger
import traceback
import sys

log = None

def getTablesFromMethod(method):
    moduleTables = []
    if hasattr(method, '__module__') and method.__module__ in sys.modules and hasattr(sys.modules[method.__module__],'tables'):
        moduleTables = sys.modules[method.__module__].tables
    return moduleTables

class CommandClr(object):
    def __init__(self, name, clr, mode='BOTH'):
        def command(self, jsonCommand):
            if mode != 'BOTH' and mode != self.mode:
                return True, []
            try:
                item = None
                for item in clr():
                    self.executor.execute(item)
                    self.simpleCommands.add(name.replace('clear','add').replace('Inventory', 'InventItem'))
                return True, []
            except Exception, ex:
                #    self.executor.rollback()
                if item:
                    log.error('%s. %s' % (ex, item))
                    return False, [item.table]
                log.error('%s' % (ex))
                return False, getTablesFromMethod(clr)
        command.__name__ = name
        setattr(Command, command.__name__, command)

class CommandAdd(object):
    def __init__(self, name, paramName, generate, delete, namedParameters = None, mode='BOTH'):
        def command(self, jsonCommand):
            if mode != 'BOTH' and mode != self.mode:
                return True, []
            param = None
            replace = jsonCommand['command'] in self.simpleCommands
            try:
                param = jsonCommand[paramName]
            except:
                log.error("The name of the parameter '%s' not found." % (paramName))
            kwargs = {}
            if namedParameters:
                for name in namedParameters:
                    val = jsonCommand.get(name, None)
                    if val:
                        kwargs[name] = val
            try:
                item = None
                if not replace:
                    for item  in delete(param, **kwargs):
                        self.executor.execute(item)
                        self.executor.sqlDelete = item
                for item in generate(param, **kwargs):
                    item.setReplace(replace)
                    self.executor.execute(item)
                    self.executor.sqlInsert = item
                    self.executor.commit()
                return True, []
            except Exception, ex:
                if item:
                    log.error('%s. %s' % (traceback.format_exc(), item))
                    return False, [item.table]
                log.error('%s' % (ex))
                #    self.executor.rollback()
                return False, getTablesFromMethod(generate)
        command.__name__ = name
        setattr(Command, command.__name__, command)

class CommandDel(object):
    def __init__(self, name, paramName, delete, namedParameters = None, mode='BOTH'):
        def command(self, jsonCommand):
            if mode != 'BOTH' and mode != self.mode:
                return True, []
            param = jsonCommand[paramName]
            kwargs = {}
            if namedParameters:
                for name in namedParameters:
                    val = jsonCommand.get(name, None)
                    if val:
                        kwargs[name] = val
            try:
                item = None
                for item  in delete(param, **kwargs):
                    self.executor.execute(item)
                    self.executor.sqlDelete = item
                self.executor.commit()
                return True, []
            except Exception, ex:
                #    self.executor.rollback()
                if item:
                    log.error('%s. %s' % (ex, item))
                    return False, [item.table]
                log.error('%s' % (ex))
                return False, getTablesFromMethod(delete)
        command.__name__ = name
        setattr(Command, command.__name__, command)

class CommandUpd(object):
    def __init__(self, name, paramName, update, namedParameters=None, mode='BOTH'):
        def command(self, jsonCommand):
            if mode != 'BOTH' and mode != self.mode:
                return True, []
            param = jsonCommand[paramName]
            kwargs = {}
            if namedParameters:
                for name in namedParameters:
                    val = jsonCommand.get(name, None)
                    if val:
                        kwargs[name] = val
            try:
                self.executor.executeMany()
                item = None
                for item  in update(param, **kwargs):
                    self.executor.execute(item)
                self.executor.commit()
                return True, []
            except Exception, ex:
                if item:
                    log.error('%s. %s' % (ex, item))
                    return False, [item.table]
                log.error('%s' % (ex))
                return False, getTablesFromMethod(update)
        command.__name__ = name
        setattr(Command, command.__name__, command)

class Command(object):
    '''
    NES command
    '''

    def __init__(self, executor, shopCode, simpleCommands = set(), mode='BOTH'):
        '''
        Constructor
        '''
        global log
        log = logger.Logger().getLogger(shopCode)
        self.executor = executor
        self.simpleCommands = simpleCommands
        self.mode = mode

    def dispatch(self, jsonCommand):
        ''' returns method which can process jsonCommand or None if no such method '''
        try:
            return getattr(self, jsonCommand['command'])
        except AttributeError:
            return None

CommandAdd(name='addTmc', paramName='invent', generate=tmc.insertTmc, delete=tmc.deleteTmc)
CommandClr(name='clearTmc', clr=tmc.clearTmc)
CommandDel(name='deleteTmc', paramName='inventcode', delete=tmc.deleteTmcByCode)

CommandAdd(name='addInventItem', paramName='invent',
             generate=invent.insertInventItem, delete=invent.deleteInventItem)
CommandClr(name='clearInventory', clr=invent.clearInventory)
CommandDel(name='deleteInventItem', paramName='inventcode', delete=invent.deleteInventItemByCode)
CommandUpd(name='updateInventItem', paramName='invent', update=invent.updateInventItem)

CommandAdd(name='addLDiscCard', paramName='ldisccard',
             generate=ldisccard.insertLDiscCard, delete=ldisccard.deleteLDiscCard)
CommandClr(name='clearLDiscCard', clr=ldisccard.clearLDiscCard)
CommandDel(name='deleteLDiscCard', paramName='cardnumber', delete=ldisccard.deleteLDiscCardByNumber)

CommandAdd(name='addLDiscFixed', paramName='ldiscfixed',
             generate=ldiscfixed.insertLDiscFixed, delete=ldiscfixed.deleteLDiscFixed)
CommandClr(name='clearLDiscFixed', clr=ldiscfixed.clearLDiscFixed)
CommandDel(name='deleteLDiscFixed', paramName='disccode', delete=ldiscfixed.deleteLDiscFixedByCode)

CommandAdd(name='addLDiscAccumulate', paramName='ldiscaccumulate',
             generate=ldiscaccumulate.insertLDiscAccumulate, delete=ldiscaccumulate.deleteLDiscAccumulate)
CommandClr(name='clearLDiscAccumulate', clr=ldiscaccumulate.clearLDiscAccumulate)
CommandDel(name='deleteLDiscAccumulate', paramName='discscheme', delete=ldiscaccumulate.deleteLDiscAccumulateByCode)

CommandAdd(name='addLDiscScheme', paramName='ldiscscheme',
            generate=ldiscscheme.insertLDiscScheme, delete=ldiscscheme.deleteLDiscScheme)
CommandClr(name='clearLDiscScheme', clr=ldiscscheme.clearLDiscScheme)
CommandDel(name='deleteLDiscScheme', paramName='schemecode', delete=ldiscscheme.deleteLDiscSchemeByCode)

CommandAdd(name='addDiscountAction', paramName='discountaction',
            generate=discountaction.insertDiscountAction, delete=discountaction.deleteDiscountAction)
CommandClr(name='clearDiscountAction', clr=discountaction.clearDiscountAction)

CommandAdd(name='addMeasure', paramName='measure',
            generate=measure.insertMeasure, delete=measure.deleteMeasure)

CommandAdd(name='addTaxGroup', paramName='taxgroup',
            generate=tax.insertTaxGroup, delete=tax.deleteTaxGroup)

CommandAdd(name='addCashUser', paramName='cashuser',
            generate=cashuser.insertCashUser, delete=cashuser.deleteCashUser)
CommandClr(name='clearCashUser', clr=cashuser.clearCashUser)
CommandDel(name='deleteCashUser', paramName='usercode', delete=cashuser.deleteCashUserByCode)

CommandAdd(name='replaceDiscountSystem', paramName='discsystemdesc',
             generate=discountsystem.replaceDiscountSystem, delete=discountsystem.deleteDiscountSystem,
             namedParameters=['limitbonussystemdesc', 'earnbonussystemdesc'])

CommandAdd(name='addUnit', paramName='unit', generate=units.insertUnit, delete=units.deleteUnit)
CommandClr(name='clearUnit', clr=units.clearUnit)
CommandDel(name='deleteUnit', paramName='unitcode', delete=units.deleteUnitByCode)

CommandAdd(name='addCard', paramName='card', generate=card.insertCard, delete=card.deleteCard)
CommandClr(name='clearCard', clr=card.clearCard)
CommandDel(name='deleteCard', paramName='idcard', delete=card.deleteCardById)
CommandUpd(name='updateCard', paramName='card', update=card.updateCard)

CommandAdd(name='addCardGroup', paramName='cardgroup', generate=cardgroup.insertCardGroup, delete=cardgroup.deleteCardGroup)
CommandClr(name='clearCardGroup', clr=cardgroup.clearCardGroup)
CommandDel(name='deleteCardGroup', paramName='idcardgroup', delete=cardgroup.deleteCardGroupById)

CommandAdd(name='addClient', paramName='client', generate=client.insertClient, delete=client.deleteClient)
CommandClr(name='clearClient', clr=client.clearClient)
CommandDel(name='deleteClient', paramName='idclient', delete=client.deleteClientById)

CommandAdd(name='addLVat', paramName='lvat', generate=lvat.insertLVat, delete=lvat.deleteLVat)
CommandClr(name='clearLVat', clr=lvat.clearLVat)
CommandDel(name='deleteLVat', paramName='code', delete=lvat.deleteLVatByCode)

CommandAdd(name='addInventGroup', paramName='inventgroup', generate=inventgroup.insertInventGroup, delete=inventgroup.deleteInventGroup)
CommandClr(name='clearInventGroup', clr=inventgroup.clearInventGroup)
CommandDel(name='deleteInventGroup', paramName='groupcode', delete=inventgroup.deleteInventGroupByCode)

CommandAdd(name='addInventCatalog', paramName='inventcatalog', generate=inventcatalog.insertInventCatalog, delete=inventcatalog.deleteInventCatalog)
CommandClr(name='clearInventCatalog', clr=inventcatalog.clearInventCatalog)
CommandDel(name='deleteInventCatalog', paramName='inventcatalog', delete=inventcatalog.deleteInventCatalog)

CommandAdd(name='addCatalog', paramName='catalog', generate=catalog.insertCatalog, delete=catalog.deleteCatalog)
CommandClr(name='clearCatalog', clr=catalog.clearCatalog)
CommandDel(name='deleteCatalog', paramName='catalogid', delete=catalog.deleteCatalogById)

CommandAdd(name='addCatalogAttribute', paramName='catalogattribute', generate=catalogattribute.insertCatalogAttribute, delete=catalogattribute.deleteCatalogAttribute)
CommandClr(name='clearCatalogAttribute', clr=catalogattribute.clearCatalogAttribute)
CommandDel(name='deleteCatalogAttribute', paramName='catalogid', delete=catalogattribute.deleteCatalogAttributeById)

CommandAdd(name='addChangeAccountBalance', paramName='changeaccountbalance', generate=changeaccountbalance.insertChangeAccountBalance, delete=changeaccountbalance.deleteChangeAccountBalance)
CommandClr(name='clearChangeAccountBalance', clr=changeaccountbalance.clearChangeAccountBalance)
CommandDel(name='deleteChangeAccountBalance', paramName='sessionid', delete=changeaccountbalance.deleteChangeAccountBalanceById)

CommandAdd(name='addCurrency', paramName='currency', generate=valut.insertValut, delete=valut.deleteValut)
CommandClr(name='clearCurrency', clr=valut.clearValut)
CommandDel(name='deleteCurrency', paramName='currencycode', delete=valut.deleteValutByCode)

CommandAdd(name='addCommand', paramName='commandobject', generate=command.insertCommand, delete=command.deleteCommand)
CommandClr(name='clearCommand', clr=command.clearCommand)
CommandDel(name='deleteCommand', paramName='commandcode', delete=command.deleteCommandByCode)

CommandAdd(name='addKbKey', paramName='kbkey', generate=kbkey.insertKbKey, delete=kbkey.deleteKbKey)
CommandClr(name='clearKbKey', clr=kbkey.clearKbKey)
CommandDel(name='deleteKbKey', paramName='keycode', delete=kbkey.deleteKbKeyByCode, namedParameters=['context'])

CommandAdd(name='addButton', paramName='button', generate=button.insertButton, delete=button.deleteButton)
CommandClr(name='clearButton', clr=button.clearButton)
CommandDel(name='deleteButton', paramName='code', delete=button.deleteButtonByCode,)

CommandAdd(name='addDept', paramName='dept', generate=dept.insertDept, delete=dept.deleteDept)
CommandClr(name='clearDept', clr=dept.clearDept)
CommandDel(name='deleteDept', paramName='code', delete=dept.deleteDeptByCode,)

CommandAdd(name='addMCashUser', paramName='mcashuser', generate=mcashuser.insertMCashUser, delete=mcashuser.deleteMCashUser)
CommandClr(name='clearMCashUser', clr=mcashuser.clearMCashUser)
CommandDel(name='deleteMCashUser', paramName='usercode', delete=mcashuser.deleteMCashUserByCode)

CommandAdd(name='addCmaction', paramName='cmaction', generate=cmaction.insertCmaction, delete=cmaction.deleteCmaction)
CommandClr(name='clearCmaction', clr=cmaction.clearCmaction)
CommandDel(name='deleteCmaction', paramName='cmactioncode', delete=cmaction.deleteCmactionByCode)

CommandAdd(name='addMenu', paramName='menu', generate=menu.insertMenu, delete=menu.deleteMenu)
CommandClr(name='clearMenu', clr=menu.clearMenu)
CommandDel(name='deleteMenu', paramName='menucode', delete=menu.deleteMenuByCode)

CommandAdd(name='addRole', paramName='role', generate=role.insertRole, delete=role.deleteRole)
CommandClr(name='clearRole', clr=role.clearRole)
CommandDel(name='deleteRole', paramName='rolecode', delete=role.deleteRoleByCode)

CommandAdd(name='addAspectScheme', paramName='aspectscheme', generate=aspectscheme.insertAspectScheme, delete=aspectscheme.deleteAspectScheme)
CommandClr(name='clearAspectScheme', clr=aspectscheme.clearAspectScheme)
CommandDel(name='deleteAspectScheme', paramName='aspectschemecode', delete=aspectscheme.deleteAspectSchemeByCode)

CommandAdd(name='addAspectValueSet', paramName='aspectvalueset', generate=aspectvalueset.insertAspectValueSet, delete=aspectvalueset.deleteAspectValueSet)
CommandClr(name='clearAspectValueSet', clr=aspectvalueset.clearAspectValueSet)
CommandDel(name='deleteAspectValueSet', paramName='aspectvaluesetcode', delete=aspectvalueset.deleteAspectValueSetByCode)

CommandAdd(name='addHotKey', paramName='hotkey', generate=hotkey.insertHotKey, delete=hotkey.deleteHotKey)
CommandClr(name='clearHotKey', clr=hotkey.clearHotKey)
CommandDel(name='deleteHotKey', paramName='hotkeycode', delete=hotkey.deleteHotKeyByCode)

CommandAdd(name='addBarcode', paramName='barcode', generate=barcode.insertBarcode, delete=barcode.deleteBarcode)
CommandClr(name='clearBarcode', clr=barcode.clearBarcode)
CommandDel(name='deleteBarcode', paramName='barcode', delete=barcode.deleteBarcodeByBarcode)

CommandAdd(name='addPrice', paramName='price', generate=price.insertPrice, delete=price.deletePrice)
CommandClr(name='clearPrice', clr=price.clearPrice)
CommandDel(name='deletePrice', paramName='barcode', delete=price.deletePriceByBarcode, namedParameters=['documentid', 'doctype'])
CommandDel(name='deletePriceByDate', paramName='barcode', delete=price.deletePriceByDate, namedParameters=['effectivedate'])

CommandAdd(name='addShop', paramName='shop', generate=shop.insertShop, delete=shop.deleteShop, mode='CASH')
CommandClr(name='clearShop', clr=shop.clearShop, mode='CASH')
CommandDel(name='deleteShop', paramName='code', delete=shop.deleteShopByCode, mode='CASH')

CommandAdd(name='addAgerestrict', paramName='agerestrict', generate=agerestrict.insertAgeRestrict, delete=agerestrict.deleteAgeRestrict)
CommandClr(name='clearAgerestrict', clr=agerestrict.clearAgeRestrict)
CommandDel(name='deleteAgerestrict', paramName='alcoholpercent', delete=agerestrict.deleteAgeRestrictByAlcoholPercent)

CommandAdd(name='addTimerestrict', paramName='timerestrict', generate=timerestrict.insertTimeRestrict, delete=timerestrict.deleteTimeRestrict)
CommandClr(name='clearTimerestrict', clr=timerestrict.clearTimeRestrict)
CommandDel(name='deleteTimerestrict', paramName='timerestrictid', delete=timerestrict.deleteTimeRestrictByTimeRestrictId)

CommandAdd(name='addTmcrequiredquantity', paramName='tmcrequiredquantity', generate=inventrequiredquantity.insertInventRequiredQuantity, delete=inventrequiredquantity.deleteInventRequiredQuantity)
CommandClr(name='clearTmcrequiredquantity', clr=inventrequiredquantity.clearInventRequiredQuantity)
CommandDel(name='deleteTmcrequiredquantity', paramName='tmccode', delete=inventrequiredquantity.deleteInventRequiredQuantity)

CommandAdd(name='addTmcvisualverify', paramName='tmcvisualverify', generate=inventvisualverify.insertInventVisualVerify, delete=inventvisualverify.deleteInventVisualVerify)
CommandClr(name='clearTmcvisualverify', clr=inventvisualverify.clearInventVisualVerify)
CommandDel(name='deleteTmcvisualverify', paramName='tmccode', delete=inventvisualverify.deleteInventVisualVerify)

CommandAdd(name='addMacro', paramName='macro', generate=macro.insertMacro, delete=macro.deleteMacro)
CommandClr(name='clearMacro', clr=macro.clearMacro)
CommandDel(name='deleteMacro', paramName='macrocode', delete=macro.deleteMacroByCode)

CommandAdd(name='addShopLabels', paramName='group', generate=shoplabels.insertShopLabels, delete=shoplabels.deleteShopLabels, mode='CASH')
CommandClr(name='clearShopLabels', clr=shoplabels.clearShopLabels, mode='CASH')
CommandDel(name='deleteShopLabels', paramName='groupcode', delete=shoplabels.deleteShopLabelsByCode, mode='CASH')

CommandAdd(name='addSoftCheck', paramName='softcheck', generate=softcheck.insertSoftCheck, delete=softcheck.deleteSoftCheck, mode='CS')
CommandClr(name='clearSoftCheck', clr=softcheck.clearSoftCheck, mode='CS')
CommandDel(name='deleteSoftCheck', paramName='softcheckid', delete=softcheck.deleteSoftCheckById, mode='CS')

# extend_objects
CommandAdd(name='addHotKeyInvent', paramName='hotkeyinvent', generate=hotkeyinvent.insertHotKeyInvent, delete=hotkeyinvent.deleteHotKeyInvent)
CommandClr(name='clearHotKeyInvent', clr=hotkeyinvent.clearHotKeyInvent)
CommandDel(name='deleteHotKeyInvent', paramName='hotkeyinvent', delete=hotkeyinvent.deleteHotKeyInvent)

CommandAdd(name='addDiscountsAuto', paramName='discountsauto', generate=discountsauto.insertDiscountsauto, delete=discountsauto.deleteDiscountsauto)
CommandClr(name='clearDiscountsAuto', clr=discountsauto.clearDiscountsauto)
CommandDel(name='deleteDiscountsAuto', paramName='discountsauto', delete=discountsauto.deleteDiscountsauto)

CommandAdd(name='addTimePeriod', paramName='timeperiod', generate=timeperiod.insertTimePeriod, delete=timeperiod.deleteTimePeriod)
CommandClr(name='clearTimePeriod', clr=timeperiod.clearTimePeriod)
CommandDel(name='deleteTimePeriod', paramName='timeperiod', delete=timeperiod.deleteTimePeriod)

CommandAdd(name='addKit', paramName='kit', generate=kit.insertKit, delete=kit.deleteKit)
CommandClr(name='clearKit', clr=kit.clearKit)
CommandDel(name='deleteKit', paramName='kit', delete=kit.deleteKit)

CommandAdd(name='addKitInvent', paramName='kitinvent', generate=kitinvent.insertKitInvent, delete=kitinvent.deleteKitInvent)
CommandClr(name='clearKitInvent', clr=kitinvent.clearKitInvent)
CommandDel(name='deleteKitInvent', paramName='kitinvent', delete=kitinvent.deleteKitInvent)

CommandAdd(name='addKitDisc', paramName='kitdisc', generate=kitdisc.insertKitDisc, delete=kitdisc.deleteKitDisc)
CommandClr(name='clearKitDisc', clr=kitdisc.clearKitDisc)
CommandDel(name='deleteKitDisc', paramName='kitdisc', delete=kitdisc.deleteKitDisc)

CommandAdd(name='addAdditionalPrice', paramName='additionalprice', generate=additionalprice.insertTprices, delete=additionalprice.deleteTprices)
CommandClr(name='clearAdditionalPrice', clr=additionalprice.clearTprices)
CommandDel(name='deleteAdditionalPrice', paramName='additionalprice', delete=additionalprice.deleteTprices)

CommandAdd(name='addCertificate', paramName='certificate', generate=certificate.insertCertificate, delete=certificate.deleteCertificate)
CommandClr(name='clearCertificate', clr=certificate.clearCertificate)
CommandDel(name='deleteCertificate', paramName='code', delete=certificate.deleteCertificateByCode)

CommandAdd(name='addSaleRestrict', paramName='salerestrict', generate=salerestrict.insertSaleRestrict, delete=salerestrict.deleteSaleRestrict)
CommandClr(name='clearSaleRestrict', clr=salerestrict.clearSaleRestrict)
CommandDel(name='deleteSaleRestrict', paramName='salerestrictcode', delete=salerestrict.deleteBySaleRestrictCode)

CommandAdd(name='addConsultant', paramName='consultant', generate=consultant.insertConsultant, delete=consultant.deleteConsultant)
CommandClr(name='clearConsultant', clr=consultant.clearConsultant)
CommandDel(name='deleteConsultant', paramName='code', delete=consultant.deleteConsultantByCode)

CommandAdd(name='addAlcoCode', paramName='alcocode', generate=alcocode.insertAlcoCode, delete=alcocode.deleteAlcoCode)
CommandClr(name='clearAlcoCode', clr=alcocode.clearAlcoCode)
CommandDel(name='deleteAlcoCode', paramName='alcocode', delete=alcocode.deleteAlcoCodeByCode)
