'''
Created on 14.04.2010

@author: gsu
'''

class _Insert(object):

    cache = {}
    cacheValues = {}

    def __init__(self, table, values, quotedValues = None, idName = None, replaceSymbol='%s', saveLastId = False):
        self.table = table
        self.values = values
        self.quotedValues = quotedValues
        self.id = None
        self.replaceSymbol = replaceSymbol
        self.saveLastId = saveLastId
        self.replace = False
        self.cacheKey = table

    def __getitem__(self, x):
        return self.values[x]

    def __setitem__(self, x, v):
        self.values[x] = v

    def __str__(self):
        return '[INSERT table=%s, values=%s, quotedValues=%s]' % (self.table, self.values, self.quotedValues)

    def setReplace(self, replace):
        self.replace = replace
        self.cacheKey = '%s_%s' % (self.table, replace)

    def prepareStatement(self, context):
        if self.cacheKey in self.cache:
            sql = _Insert.cache[self.cacheKey]
        else:
            sqlCommand = 'replace' if self.replace else 'insert ignore' if context.executeMany else 'insert'
            tableName = context.tableName(self.table)
            fields = self.values.keys()
            fieldsCount = len(fields)
            if self.quotedValues != None:
                quotedFields = self.quotedValues.keys()
                sql = '%s into %s (`%s`, `%s`) values (%s, %s)' % (sqlCommand, tableName, '`,`'.join(fields),
                            ','.join(quotedFields),'`,`'.join([self.replaceSymbol]*fieldsCount),
                            ','.join(self.quotedValues.values()))
            else:
                sql = '%s into %s (`%s`) values (%s)' % (sqlCommand, tableName, '`,`'.join(fields),
                                                   ','.join([self.replaceSymbol]*fieldsCount))
            _Insert.cache[self.cacheKey] = sql
            _Insert.cacheValues[self.cacheKey] = []
        return sql

    def execute(self, context):
        values = self.values.values()
        sql = self.prepareStatement(context)
        if context.executeMany and not self.saveLastId:
            _Insert.cacheValues[self.cacheKey].append([ v() if callable(v) else v for v in values])
        else:
            context.cursor.execute(sql, [ v() if callable(v) else v for v in values])
            if self.saveLastId:
                context.cursor.execute('SELECT LAST_INSERT_ID()')
                for value in context.cursor:
                    self.id = value[0]

    def execMany(self, context):
        for table in _Insert.cache.keys():
            context.cursor.executemany(_Insert.cache[table], _Insert.cacheValues[table])
            del _Insert.cacheValues[table][:]

    def lastId(self):
        return self.id;

class _Update(object):

    cache = {}

    def __init__(self, table, fields, values, whereField, whereValue, replaceSymbol='%s'):
        self.table = table        
        self.fields = fields
        self.values = values
        self.whereField = whereField
        self.whereValue = whereValue
        self.replaceSymbol = replaceSymbol
        
    def __str__(self):
        return '[UPDATE table=%s, fields=%s, values=%s, whereField=%s, whereValue=%s]' % (self.table, self.fields, self.values, self.whereField, self.whereValue)
    
    def prepareStatement(self, context):
        fields = self.fields
        values = self.values
        whereField = self.whereField
        whereValue = self.whereValue
        tableName = context.tableName(self.table)
        setClause = (', '.join(["%s = '?'"] * len(fields)) % tuple(fields)).replace('?', '%s')
        setClause = setClause % tuple(values)
        sql = ('update %s set %s where %s in (?)' % (tableName, setClause, whereField)).replace('?', self.replaceSymbol)
        return sql
    
    def execute(self, context):
        sql = self.prepareStatement(context)
        if isinstance(self.whereValue, _Select):
            whereValue = map(lambda x: list(x), self.whereValue.execute(context))
            context.cursor.executemany(sql, whereValue)
        else:
            whereValue = list(self.whereValue)
            context.cursor.execute(sql, whereValue)
        
class _LastInsertId(object):

    value = None
    
    def __init__(self):
        self.value = None

    def __str__(self):
        return '[SELECT LAST_INSERT_ID()]'

    def execute(self, context):
        if self.value != None:
            return self.value
        sql = 'SELECT LAST_INSERT_ID()'
        context.cursor.execute(sql)
        for value in context.cursor:
            self.value = value[0]
        return self.value

    def id(self):
        return lambda : self.value;

class _Select(object):

    result = False
    cache = {}

    def __init__(self, table, fields, whereFields, whereValues, replaceSymbol='%s'):
        self.table = table
        self.fields = fields
        self.whereFields = whereFields
        self.whereValues = whereValues
        self.result = False
        self.replaceSymbol = replaceSymbol

    def __str__(self):
        return '[SELECT table=%s, fields=%s, whereFields=%s, whereValues=%s]' % (self.table, self.fields, self.whereFields, self.whereValues)
    
    def prepare(self, context):
        if self.table in _Select.cache:
            return _Select.cache[self.table]
        tableName = context.tableName(self.table)
        whereFields = self.whereFields  
        whereClause = (' and '.join(['%s = ?'] * len(whereFields)) % tuple(whereFields)).replace('?', self.replaceSymbol)
        sql = 'select %s from %s where %s' % (','.join(self.fields), tableName, whereClause)
        _Select.cache[self.table] = sql      
        return sql
            
    def execute(self, context):
        if self.result != False:
            return self.result
        sql = self.prepare(context)
        if isinstance(self.whereValues, _Select):
            whereValues = map(lambda x: list(x), self.whereValues.execute(context))
            result = []
            for val in whereValues:
                context.cursor.execute(sql, val)
                result += context.cursor.fetchall()
        else:
            whereValues = [ str(val.encode("utf8") if isinstance(val,unicode) else val) for val in list(self.whereValues) ]
            context.cursor.execute(sql, whereValues)
            result = context.cursor.fetchall()
        self.result = result        
        return result
    
    def oneValue(self):
        if self.result != None:
            return self.result[0][0]
        else:
            return None

class _Delete(object):

    cache = {}
    cacheValues = {}

    def __init__(self, table, whereFields, whereValues, replaceSymbol='%s', notExecMany = False):
        self.table = table
        self.whereFields = whereFields
        self.whereValues = whereValues
        self.replaceSymbol = replaceSymbol
        self.notExecMany = notExecMany

    def __str__(self):
        return '[DELETE table=%s, whereFields=%s, whereValues=%s]' % (self.table, self.whereFields, self.whereValues)

    def prepareStatement(self, context):
        hashkey = self.table+'@'+repr(self.whereFields)
        if hashkey in _Delete.cache:
            sql = _Delete.cache[hashkey]
        else:
            tableName = context.tableName(self.table)
            whereFields = self.whereFields
            whereClause = (' and '.join(['%s = ?'] * len(whereFields)) % tuple(whereFields)).replace('?', self.replaceSymbol)
            sql = 'delete from %s where %s' % (tableName, whereClause)
            _Delete.cache[hashkey] = sql
            _Delete.cacheValues[hashkey] = []
        return sql, hashkey
    
    def execute(self, context):
        if self.table in context.emptyTables:
            return
        sql, hashkey = self.prepareStatement(context)
#        context.cursor.execute('FLUSH tables;')
        if isinstance(self.whereValues, _Select):
            whereValues = map(lambda x: list(x), self.whereValues.execute(context))
            if len(whereValues) > 0:
                if context.executeMany and not self.notExecMany:
                    _Delete.cacheValues[hashkey].extend(whereValues)
                else:
                    context.cursor.executemany(sql, whereValues)
        else:
            whereValues = [ str(val.encode("utf8") if isinstance(val,unicode) else val) for val in list(self.whereValues) ]
            if context.executeMany and not self.notExecMany:
                _Delete.cacheValues[hashkey].append(whereValues)
            else:
                context.cursor.execute(sql, whereValues)

    def execMany(self, context):
        for key in _Delete.cache.keys():
            context.cursor.executemany(_Delete.cache[key], _Delete.cacheValues[key])
            del _Delete.cacheValues[key][:]

class _DeleteAll(object):
    
    def __init__(self, table):
        self.table = table

    def __str__(self):
        return "[DELETEALL table=%s]" % (self.table,)
    
    def execute(self, context):
        tableName = context.tableName(self.table)
        sql = "truncate %s " % tableName
        context.cursor.execute(sql)
        context.emptyTables[self.table] = True
    
    def execMany(self, context):
        self.execute(context)
