# -*- coding: UTF-8 -*-

import sys, os
from qpid.messaging import Connection, Empty, NotFound
from qmf.console import Session
import ConfigParser, time
import glob
import marshal
import atexit
import tempfile
import traceback
import threading
import json
import db_inserter
import db_request_executor
from bottle import route, run, response
from httplib import INTERNAL_SERVER_ERROR
from nes_common import logger, rest_client
from nes_common.basictaskrunner import BasicTaskRunner
from nes_common.mproctaskrunner import MprocTaskRunner
from _mysql_exceptions import OperationalError, DatabaseError

serviceLog = logger.Logger().getLogger('Service')

lockFile = os.path.normpath('%s/locks/lockQueueGetter' % tempfile.gettempdir())
prefixQueueName = 'artix.loaddict.'
taskRunner = None
timeShopsReread = 0

def unlock():
    if os.path.exists(lockFile):
        serviceLog.info('remove lock file %s' % lockFile)
        os.remove(lockFile)

atexit.register(unlock)

def getShopDests(host, port, user, password, brokerType='cpp', brokerRestPort=None):
    serviceLog.debug('Получаем список магазинов...')
    shopDests = []
    if brokerType == 'java':
        queueNames = rest_client.getQueueNames(host, brokerRestPort, user, password)
        for name in queueNames:
            if name.startswith(prefixQueueName):
                shopDests.append(name.replace(prefixQueueName, ''))
    else:
        sess = Session()
        broker = sess.addBroker(target='%s/%s@%s:%s' % (user, password, host, port))
        queues = sess.getObjects(_class="queue", _package="org.apache.qpid.broker")
        for q in queues:
            if q.name.startswith(prefixQueueName):
                shopDests.append(q.name.replace(prefixQueueName, ''))
        sess.close()
    serviceLog.debug('shopDests = %s' % shopDests)
    return shopDests

@route('/nes/queue_reread', method='PUT')
def restQueueReread():
    global timeShopsReread
    response.content_type = 'application/json'
    result = {u'success': False}
    try:
        timeShopsReread = 0
        serviceLog.info(u'Change timeShopsReread =  %s' % timeShopsReread)
        result = {u'success': True}
    except Exception, e:
        serviceLog.warning(u'Ошибка REST сервиса. %s' % str(e))
        serviceLog.warning(traceback.format_exc())
        response.status = INTERNAL_SERVER_ERROR
        result = {u'success': False}
    return json.dumps(result, ensure_ascii=False)

def dictsCountCommandToStr(countGoodCommands, countBadCommands, command, ruName):
    if countGoodCommands[command] + countBadCommands[command] > 0:
        s_ru = 'Комманды %s: успешно - %s, ошибок - %s; ' % (ruName, countGoodCommands[command], countBadCommands[command])
        s_en = '%s commans: ok - %s, error - %s; ' % (command, countGoodCommands[command], countBadCommands[command])
        return s_ru, s_en
    return '', ''

def dictsCountCommandsToStr(countGoodCommands, countBadCommands):
    clear_ru, clear_en = dictsCountCommandToStr(countGoodCommands, countBadCommands, 'clear', 'очистки')
    add_ru, add_en = dictsCountCommandToStr(countGoodCommands, countBadCommands, 'add', 'добавления')
    delete_ru, delete_en = dictsCountCommandToStr(countGoodCommands, countBadCommands, 'delete', 'удаления')
    update_ru, update_en = dictsCountCommandToStr(countGoodCommands, countBadCommands, 'update', 'обновления')
    return clear_ru + add_ru + delete_ru + update_ru, clear_en + add_en + delete_en + update_en

def receiveMessages(shopDest, broker):
    shop = {'shopId': shopDest.split('.')[0]}
    shop['clusterId'] = shopDest.replace(shop['shopId'] + '.', '') if shop['shopId'] != shopDest else ''
    log = logger.Logger().getLogger(shop['shopId'])
    log.debug('Connect to broker = %s' % broker)
    connection = Connection(broker)
    connection.open()
    session = connection.session()
    receiver = None
    try:
        receiver = session.receiver('%s%s' % (prefixQueueName, shopDest))
    except NotFound:
        log.info('Очередь %s%s еще не создана.' % (prefixQueueName, shopDest))
        connection.close()
        return True
    dictLoadStatusId = None
    cashCode = None
    statusServer = 'localhost:8080'
    loadDictStatusSender = None
    dictBackOfficeIds = set()
    while True:
        try:
            message = receiver.fetch(0)
            if message.properties.get('type') == 'control' and message.properties.get('control') == 'begin_tran':
                log.info('Начало транзакции...')
                dictCSStatusId = message.properties.get('dictBackOfficeId')
                if dictCSStatusId:
                    dictBackOfficeIds.add(dictCSStatusId)

                dictCashStatusId = message.properties.get('dictCashLoadStateId')
                cashCode = message.properties.get('cashCode')
                statusServer = message.properties.get('statusServer')

                version = message.properties.get('version')
                loadDictStatusSender = rest_client.LoadDictStatusSender(shop['clusterId'] + shop['shopId'], dictCSLoadStatusId=dictCSStatusId, dictCashLoadStatusId=dictCashStatusId,
                                                                        statusServer=statusServer, cashCode=cashCode, version=version)
                try:
                    inserter = db_inserter.DB_Inserter(shop, loadDictStatusSender)
                except (DatabaseError, OperationalError) as e:
                    log.error('Ошибка подключения к БД. %s' % e)
                    log.info('Попробуем выполнить загрузку позже.')
                    connection.close()
                    return True
                log.info('Подготовка к загрузке справочников...')
                inserter.start(message.properties.get('startMessageId'))
                typeMessage = None
                log.info('Просматриваем сообщения...')
                loadObjects = False
                tableNames = set()
                countGoodCommands = {'add': 0, 'delete': 0, 'clear': 0, 'update': 0}
                countBadCommands = {'add': 0, 'delete': 0, 'clear': 0, 'update': 0}
                while typeMessage != 'control':
                    try:
                        message = receiver.fetch()
                    except Empty:
                        loadDictStatusSender.sendDictState('Not found message "end transaction"', 'Не найдено сообщение "конец транзакции"', rest_client.STATUS_ERROR)
                        log.error("Не найдено сообщение 'конец транзакции'")
                        return False
                    typeMessage = message.properties.get('type')
                    if typeMessage == 'datagram':
                        if message.properties.get('serialType') == 'marshal':
                            dictObjects = marshal.loads(message.content)
                            log.info('Добавляем объекты в БД')
                            try:
                                tables, countGoodCommandsInPackage, countBadCommansInPackage = inserter.add_package(dictObjects)
                                for t in tables:
                                    tableNames.add(t)
                                for key in countGoodCommands.keys():
                                    countGoodCommands[key] += countGoodCommandsInPackage[key]
                                    countBadCommands[key] += countBadCommansInPackage[key]
                            except Exception, e:
                                log.error('Ошибка при добавлении объектов на загрузку в БД: %s' % e)
                                inserter.flagUnBlockSet()
                                connection.close()
                                loadDictStatusSender.sendDictState(e.message, 'Ошибка: %s' % e.message, rest_client.STATUS_ERROR)
                                return False
                            loadObjects = True
                    elif typeMessage != 'control':
                        log.warning('Получили сообщение неизвестного формата.')
                if not loadObjects:
                    log.warning('Не получено ни одного сообщения с объектами из справочника.')
                log.info('Закончили обход объектов в текущей транзакции')
                if message.properties.get('type') == 'control' and message.properties.get('control') == 'end_tran':
                    result = inserter.stop()
                    if not result:
                        loadDictStatusSender.sendDictState('Stop failure insert into DB', 'Ошибка завершения вставки в БД', rest_client.STATUS_ERROR)
                        return result
                    session.acknowledge()
                    log.info('Транзакция для магазина - %s окончена.' % shop['shopId'])
                    mes_ru, mes_en = dictsCountCommandsToStr(countGoodCommands, countBadCommands)
                    if sum(countBadCommands.values()) > 0:
                        loadDictStatusSender.sendDictState('Dictionary is not fully loaded. ' + mes_en, 'Справочник загружен, но не полностью. Были ошибки. ' + mes_ru,
                                                           rest_client.STATUS_READY_WITH_ERROR, tableNames)
                        if not loadDictStatusSender.cashCode:  # при загрузке в КС для остальных таблиц, у которых была успешная загрузка ставим "хороший" статус
                            loadDictStatusSender.sendDictState('OK ' + mes_en, 'ОК ' + mes_ru, rest_client.STATUS_READY)
                    else:
                        loadDictStatusSender.sendDictState('OK ' + mes_en, 'ОК ' + mes_ru, rest_client.STATUS_READY)
                else:
                    log.error("Не найдено сообщение 'конец транзакции'")
                    loadDictStatusSender.sendDictState('Not found message "end transaction"', 'Не найдено сообщение "конец транзакции"', rest_client.STATUS_ERROR)
                    return False
            elif message.properties.get('type') == 'comparetable':
                request = message.properties.get('requestCash')
                compareName = message.properties.get('compareName')
                compareResultServerId = message.properties.get('compareResultServerId')
                statusServer = message.properties.get('statusServer')
                cashCode = message.properties.get('cashCode')
                log.info(u"Получено сообщение для сверки данных <%s>. request: %s" % (compareName, request))
                log.debug(u"Доп. пораметры: compareResultServerId = %s, statusServer = %s, cashId = %s" 
                          % (compareResultServerId, statusServer, cashCode))
                requestExecutor = db_request_executor.DB_RequestExecutor(shop)
                log.info("Выполняем запрос в БД")
                executeResult = requestExecutor.execute(request)
                resultServer = message.content
                if resultServer != None:
                    log.debug(u"Резальтат выполнения запроса на КС: %s" % (unicode(resultServer, 'utf-8')))
                log.debug(u"Резальтат выполнения запроса: %s" % (str(executeResult).decode('string-escape').decode('utf8')))
                log.info("Отправляем результат запроса на сервер.")
                rest_client.sendCompareTableResult(compareResultServerId, shop.get('shopId', ''), cashCode, executeResult, statusServer)
                session.acknowledge()
            else:
                log.warning('Получили сообщение неизвестного формата.')
                session.acknowledge()
        except Empty:
            log.debug('Проверка сообщений для магазина - %s закончена' % shop['shopId'])
            if len(dictBackOfficeIds) > 0:
                log.info('Запускаем генерацию справоников на КС')
                rest_client.generateDict(shop['shopId'], dictBackOfficeIds)
            connection.close()
            return True
        except Exception, e:
            log.error('Ошибочка: %s' % traceback.format_exc())
            connection.close()
            if loadDictStatusSender:
                loadDictStatusSender.sendDictState(str(e), 'Ошибка: %s' % e, rest_client.STATUS_ERROR)
            return False
    return True

def getIntFromConfig(config, nameOfParam, defaultValue, section='QUEUE'):
    return config.getint(section, nameOfParam) if config.has_section(section) and config.has_option(section, nameOfParam) else defaultValue

def getStrFromConfig(config, nameOfParam, defaultValue, section='QUEUE'):
    return config.get(section, nameOfParam) if config.has_section(section) and config.has_option(section, nameOfParam) else defaultValue

def startRestServiceForCS(config, isCash):
    if not isCash:
        restServiceHost = getStrFromConfig(config, 'host', '0.0.0.0', 'REST_SERVICE')
        restServicePort = getIntFromConfig(config, 'port', 28081, 'REST_SERVICE')
        try:
            serviceLog.debug('runing REST service on %s:%s...' % (restServiceHost, restServicePort))
            t = threading.Thread(target=run, kwargs={'host':restServiceHost, 'port':restServicePort})
            t.daemon = True
            t.start()
        except Exception, e:
            serviceLog.warning(u'Ошибка запуска REST сервиса: %s' % e.message)
            serviceLog.warning(traceback.format_exc())

def start(service=True, isCash=True):
    global taskRunner
    global timeShopsReread
    taskRunner = BasicTaskRunner() if isCash else MprocTaskRunner()
    if service and os.path.exists(lockFile):
        serviceLog.debug('Exist lock file %s' % lockFile)
        serviceLog.error("Can not run service. This service is runing.")
        os._exit(1)  # выход без выполнения atexit
    config = ConfigParser.RawConfigParser()
    if not config.read(sys.path[0] + "/../../../config/nes_load.ini"):
        serviceLog.error("Configuration file 'nes_load.ini' not found.")
        sys.exit(1)
    poolProcessSize = config.getint('MAIN', 'poolProcessSize')
    host = getStrFromConfig(config, 'brokerHost', 'localhost')
    port = getIntFromConfig(config, 'brokerPort', 5672)
    user = getStrFromConfig(config, 'user', 'guest')
    password = getStrFromConfig(config, 'password', 'guest')
    brokerType = getStrFromConfig(config, 'brokerType', 'cpp')
    brokerRestPort = getIntFromConfig(config, 'brokerRestPort', 9080)
    broker = '%s/%s@%s:%s' % (user, password, host, port)
    nesProgressFile = "/linuxcash/cash/exchangesystems/progress/nes.json"
    countProgressFile = "/linuxcash/cash/exchangesystems/progress/counter_*.json"
    serviceLog.debug('service mode = %s' % service)
    serviceLog.debug('isCash = %s' % isCash)
    if service:
        startRestServiceForCS(config, isCash)
        if not os.path.exists(os.path.dirname(lockFile)):
            os.makedirs(os.path.dirname(lockFile))
        f = open(lockFile, 'w')
        f.close()


    results = {}
    shopDests = None
    taskRunner.init(poolProcessSize)
    serviceLog.debug('Start monitoring...')
    forever = True
    while forever:
        forever = service

        if isCash:
            shopDests = ['cash']
            connection = Connection(broker)
            connection.open()
            session = connection.session()
            try:
                receiver = session.receiver('%s%s' % (prefixQueueName, 'cash'))
                message = receiver.fetch(0)
            except (Empty, NotFound):
                try:
                    if os.path.exists(nesProgressFile):
                        os.remove(nesProgressFile)
                except Exception, e:
                    serviceLog.error(u'Ошибка удаления %s. %s' % (nesProgressFile, e))

                try:
                    for file in glob.glob(countProgressFile):
                        os.remove(file)
                except Exception, e:
                    serviceLog.error(u'Ошибка удаления %s. %s' % (countProgressFile, e))
            finally:
                connection.close()
        else:
            if timeShopsReread == 0 or (time.time() - 600) > timeShopsReread:
                timeShopsReread = time.time()
                shopDests = getShopDests(host, port, user, password, brokerType, brokerRestPort)

        for shopDest in shopDests:
            taskRunner.addAsyncTask(shopDest, receiveMessages, [shopDest, broker])

        if service:
            time.sleep(20)

def stop():
    serviceLog.info('Stoping...')
    if taskRunner:
        taskRunner.stopAllTasks()
    serviceLog.info('Exit...')
    exit()
